﻿import { Component, OnInit, TemplateRef } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { Router } from '@angular/router';
import { Entry, Message } from '../models';
import { AppConfigService, EntryService, MessageService } from '../services';
import { BsModalService } from 'ngx-bootstrap/modal';
import { BsModalRef } from 'ngx-bootstrap/modal/modal-options.class';
import { EntryDeleteComponent } from '../entry-delete/entry-delete.component';
import { ServerErrorComponent } from '../server-error/server-error.component';

@Component({
  selector: 'app-entry-list',
  templateUrl: './entry-list.component.html',
  styleUrls: ['./entry-list.component.css']
})
export class EntryListComponent implements OnInit {
  title: string = "Entries";
  entries: Entry[] = [];
  averageDailyActivity: number = 0;
  totalActivity: number = 0;
  isLoadingData: boolean = false;

  constructor(
    private appConfigService: AppConfigService,
    private entryService: EntryService, 
    private messageService: MessageService,
    private modalService: BsModalService,
    private router: Router) { }

  ngOnInit() {
    this.isLoadingData = true;

    this.entryService.getList()
      .subscribe(data => {
        this.entries = data;
        this.calculateTotalAndAverageDailyActivity(data);
        this.isLoadingData = false;
      }, (err: HttpErrorResponse) => {
        const modalRef = this.modalService.show(ServerErrorComponent);
        const serverErrorComponent = <ServerErrorComponent>modalRef.content;
        serverErrorComponent.error = err;
      });
  }

  editCompleted(entry: Entry) {
    // Update the entries collection with the updated entry.
    const entryIndex = this.entries.findIndex(e => e.id === entry.id);
    this.entries[entryIndex] = entry;

    // Sort the entries in case the entry date was updated.
    Entry.sortEntries(this.entries);

    // Display a message.
    this.messageService.displayMessage('Your entry was successfully updated!');

    // Recalculate the total and average daily activity.
    this.calculateTotalAndAverageDailyActivity(this.entries);    
  }

  confirmDeletion(entry: Entry) {
    const modalRef = this.modalService.show(EntryDeleteComponent);
    const entryDeleteComponent = <EntryDeleteComponent>modalRef.content;
    entryDeleteComponent.displayEntry(entry);
    entryDeleteComponent.onEntryDeleted.subscribe(() => {
      // Remove the entry from the entries collection.
      this.entries.splice(this.entries.indexOf(entry), 1);

      // Display a message.
      this.messageService.displayMessage('Your entry was successfully deleted!');

      // Recalculate the total and average daily activity.
      this.calculateTotalAndAverageDailyActivity(this.entries);
    });
  }

  private calculateTotalAndAverageDailyActivity(entries: Entry[]) {
    this.totalActivity = this.calculateTotalActivity(entries);
    this.averageDailyActivity = this.calculateAverageDailyActivity(entries, this.totalActivity);
  }

  private calculateAverageDailyActivity(entries: Entry[], totalActivity: number): number {
    // Strip off all of the time values.
    const datesWithoutTime = entries.map(e => {
      let date: Date;

      if (e.date instanceof Date) {
        date = e.date;
      } else {
        date = new Date(e.date);
      }
      
      return date.toLocaleDateString();
    });

    // Use a Set to determine the unique dates.
    const uniqueDates = new Set(datesWithoutTime);

    // Use the size of the Set to determine the number of active days.
    const numberOfActiveDays = uniqueDates.size;

    // Return the average daily activity.
    return (totalActivity / numberOfActiveDays);
  }

  private calculateTotalActivity(entries: Entry[]): number {
    return entries.reduce((totalActivity, entry) => {
      // Only add the entry's duration to the total activity if it's not excluded.
      if (!entry.exclude) {
        totalActivity += entry.duration;
      }
      return totalActivity;
    }, 0);
  }
}
